import javax.swing.*;
import java.awt.*;

import java.io.*;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Scanner;
/**
 * ChessGameGUI is a class where we set the size and positi of the frames
 *@author kimiadorani
 *@version 1.0
 *@since 2019-5-7
 */


public class ChessGameGUI extends JFrame {
    private String URL ="";
    private PrintWriter out;
    private Scanner input;
    private ChessBoardGUI gameBoard;
    public static void main(String args[]){
        new ChessGameGUI();
    }

    public ChessGameGUI(){
        super();
        URL = JOptionPane.showInputDialog("Insert Server Address (Leave Blank if want be Server)");
        if(URL.isEmpty()){
            server();
            ChessBoardGUI.isServer = true;
            setTitle("Server: Player 1(White)");
        }
        else {
            client(URL);
            setTitle("Client: Player 2(Black)");
        }
        JPanel parent = new JPanel();
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setSize(1300,610);
        parent.setSize(1300,610);
        setResizable(false);
        parent.setLayout(new BoxLayout(parent,BoxLayout.X_AXIS));
        //-----------------------------------------
        JPanel whiteRemovePanel = new JPanel();
        whiteRemovePanel.setLayout(new GridLayout(8,2));
        whiteRemovePanel.setSize(200,600);
        //-----------------------------------------
        JPanel turnPanel = new JPanel();
        //-----------------------------------------
        JPanel blackRemovePanel = new JPanel();
        blackRemovePanel.setLayout(new GridLayout(8,2));
        blackRemovePanel.setSize(200,600);
        //-----------------------------------------

        gameBoard = new ChessBoardGUI(whiteRemovePanel,blackRemovePanel,turnPanel,this,out);
        gameBoard.setSize(800,600);
        parent.add(whiteRemovePanel);
        parent.add(turnPanel);
        parent.add(blackRemovePanel);
        parent.add(gameBoard);
        add(parent);
        setVisible(true);
    }

    /**
     * creating a socket and waits for a client to connect with 1234 port . as soon as
     * a client connected to the socket it gets an Stream as an input(inputStream) and output as an output(outputStream).
     */
    private void server(){
        try {
            ServerSocket server = new ServerSocket(1234);

            Socket socket = server.accept();
            System.out.println("Server Socket Completed");
            input = new Scanner(socket.getInputStream());
            out = new PrintWriter(socket.getOutputStream());

            Thread thread = new Thread() {
                @Override
                public void run() {
                    while(input.hasNextLine()){
                        String str = input.nextLine();
                        System.out.println("Client Message:"+str);
                        String splt[] = str.split("\t");
                        Cell source = new Cell(Integer.parseInt(splt[0]),Integer.parseInt(splt[1]));
                        Cell dest = new Cell(Integer.parseInt(splt[2]),Integer.parseInt(splt[3]));
                        if(gameBoard.movePiece(source,dest)) {
                            gameBoard.updatesAfterMove();
                        }
                    }
                }
            };
            thread.start();

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * try to connect to the socket server and gets an inputStream and outputStream
     * we write changes of the games on outputStream and send it to other app
     * we read inputs of inputStream then change the game
     * @param address address is an input we give to the system which is the address of the server
     */
    private void client(String address){
        try {
            Socket socket = new Socket(address,1234);
            System.out.println("Client Socket Completed");
            out = new PrintWriter(socket.getOutputStream());
            input = new Scanner(socket.getInputStream());
            out.flush();
            Thread thread = new Thread() {
                @Override
                public void run() {
                    while(input.hasNextLine()){
                        String str = input.nextLine();
                        System.out.println("Server Message: "+str);
                        String splt[] = str.split("\t");
                        Cell source = new Cell(Integer.parseInt(splt[0]),Integer.parseInt(splt[1]));
                        Cell dest = new Cell(Integer.parseInt(splt[2]),Integer.parseInt(splt[3]));
                        if(gameBoard.movePiece(source,dest)) {
                            gameBoard.updatesAfterMove();
                        }
                    }
                }
            };
            thread.start();
        } catch (IOException e) {
            e.printStackTrace();
        }

    }
}
